package org.apache.maven.linkcheck.validation;

/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2003 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 *    "Apache Maven" must not be used to endorse or promote products
 *    derived from this software without prior written permission. For
 *    written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    "Apache Maven", nor may "Apache" appear in their name, without
 *    prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * ====================================================================
 */

import org.apache.commons.httpclient.HostConfiguration;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpState;
import org.apache.commons.httpclient.UsernamePasswordCredentials;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Checks links which are normal URLs
 * @author <a href="mailto:bwalding@apache.org">Ben Walding</a>
 * @version $Id: HTTPLinkValidator.java,v 1.6 2003/03/10 14:20:21 bwalding Exp $
 */
public class HTTPLinkValidator implements LinkValidator
{
    /**
     * Log for debug output
     */
    private static Log LOG = LogFactory.getLog(HTTPLinkValidator.class);

    private static final LinkValidationResult LVR_INVALID =
        new LinkValidationResult(LinkValidationResult.INVALID, true);

    private static final LinkValidationResult LVR_VALID = new LinkValidationResult(LinkValidationResult.VALID, true);

    private boolean proxy;
    private String proxyHost;
    private int proxyPort;
    private String proxyUser;
    private String proxyPass;

    public HTTPLinkValidator(String proxyHost, String proxyPort, String proxyUser, String proxyPass)
    {
        if (proxyHost == null)
        {
            proxy = false;
            LOG.debug("Not using a proxy");
        }
        else
        {
            proxy = true;
            this.proxyHost = proxyHost;
            this.proxyPass = proxyPass;
            this.proxyUser = proxyUser;
            this.proxyPort = Integer.parseInt(proxyPort);
            LOG.debug("Using Proxy Host:" + proxyHost);
            LOG.debug("Using Proxy Port:" + proxyPort);
            LOG.debug("Using Proxy User:" + proxyUser);
        }
    }

    /**
     * @see org.apache.maven.linkcheck.LinkValidator#validateLink(org.apache.maven.linkcheck.LinkValidationItem)
     */
    public LinkValidationResult validateLink(LinkValidationItem lvi)
    {
        try
        {
            String link = lvi.getLink();
            LOG.debug("Checking web link:" + link);

            HttpClient cl = new HttpClient();
            HostConfiguration hc = new HostConfiguration();

            if (proxyHost != null)
            {
                hc.setProxy(proxyHost, proxyPort);
            }
            HttpState state = new HttpState();

            if (proxyUser != null && proxyPass != null)
            {
                state.setProxyCredentials(null, new UsernamePasswordCredentials(proxyUser, proxyPass));
            }

            GetMethod get = new GetMethod(link);
            cl.setState(state);
            get.setFollowRedirects(true);

            // execute the GET
            int status = 404;
            try
            {
                status = cl.executeMethod(get);
            }
            catch (Exception e)
            {
                System.out.println(e);
            }

            //FIXME: This constant is defined somewhere, but I can't remember where...
            if (status == 200)
            {
                return LVR_VALID;
            }
            else
            {
                String msg = "Received: [" + status + "] for " + link;
                LOG.info(msg);
                System.out.println(msg);
                return LVR_INVALID;
            }

        }
        catch (Exception e)
        {
            LOG.warn("Error accessing " + lvi.getLink(), e);
            e.printStackTrace();
            return LVR_INVALID;
        }

    }

    /**
     * @see org.apache.maven.linkcheck.LinkValidator#getResourceKey(org.apache.maven.linkcheck.LinkValidationItem)
     */
    public Object getResourceKey(LinkValidationItem lvi)
    {
        String link = lvi.getLink();

        if (!link.startsWith("http://"))
            return null;

        int hashPos = link.indexOf("#");
        if (hashPos != -1)
            link = link.substring(0, hashPos);

        return link;
    }

}
